@@ -3,7 +3,8 @@ import torch
 import warnings
 from pdb import set_trace
 from scipy.optimize import linear_sum_assignment
-from dl_utils.tensor_funcs import numpyify
+from scipy import stats
+from dl_utils.tensor_funcs import numpyify, recursive_np_or
 
 
 class TranslationError(Exception):
@@ -18,12 +19,13 @@ def unique_labels(labels):
     else:
         print("Unrecognized type for labels:", type(labels))
         raise TypeError
+
 def label_assignment_cost(labels1,labels2,label1,label2):
     if len(labels1) != len(labels2):
         raise TranslationError(f"len labels1 {len(labels1)} must equal len labels2 {len(labels2)}")
     return len([idx for idx in range(len(labels2)) if labels1[idx]==label1 and labels2[idx] != label2])
 
-def get_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
+def get_trans_dict(trans_from_labels,trans_to_labels,subsample_size='none'):
     # First compress each labelling, retain compression dicts
     trans_from_labels, tdf, _ = compress_labels(trans_from_labels)
     trans_to_labels, tdt, _ = compress_labels(trans_to_labels)
@@ -46,7 +48,7 @@ def get_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
     trans_dict = {reverse_tdf[k]:reverse_tdt[v] for k,v in trans_dict.items()}
     return trans_dict,leftovers
 
-def translate_labellings(trans_from_labels,trans_to_labels,subsample_size):
+def translate_labellings(trans_from_labels,trans_to_labels,subsample_size='none'):
     if trans_from_labels.shape != trans_to_labels.shape:
         raise TranslationError(f"trans_to_labels: {trans_to_labels.shape} doesn't equal trans_to_labels shape: {trans_from_labels.shape}")
     if len(trans_from_labels) == 0:
@@ -188,3 +190,41 @@ def dummy_labels(num_classes,size):
     combined = np.concatenate((main_chunk,extra_chunk), axis=0)
     assert combined.shape[0] == size
     return combined
+
+def acc_by_label(labels1, labels2, subsample_size='none'):
+    labels1 = translate_labellings(labels1,labels2,subsample_size)
+    acc_by_labels_from = {}
+    for label in np.unique(labels1):
+        label_preds = labels2[labels1==label]
+        num_correct = (label_preds==label).sum()
+        total_num = len(label_preds)
+        acc_by_labels_from[label] = round(num_correct/total_num,4)
+    acc_by_labels_to = {}
+    for label in np.unique(labels2):
+        label_preds = labels1[labels2==label]
+        num_correct = (label_preds==label).sum()
+        total_num = len(label_preds)
+        acc_by_labels_to[label] = round(num_correct/total_num,4)
+    return acc_by_labels_from, acc_by_labels_to
+
+def avoid_minus_ones_lf_wrapper(lf):
+    def wrapped_lf(pred,target,multiplicative_mask='none'):
+        avoidance_mask = target!=-1
+        loss_array = lf(pred[avoidance_mask],target[avoidance_mask])
+        if multiplicative_mask is not 'none':
+            loss_array *= multiplicative_mask[avoidance_mask]
+        return loss_array.mean()
+    return wrapped_lf
+
+def select_by_label(labelling,labels_to_select_by):
+    return recursive_np_or([labelling==lab for lab in labels_to_select_by])
+
+def true_cross_entropy_with_logits(pred,target):
+    return (-(pred*target).sum(dim=1) + torch.logsumexp(pred,dim=1)).mean()
+
+def masked_mode(pred_array,mask='none'):
+    if mask is 'none':
+        return stats.mode(pred_array,axis=0).mode[0]
+    x = mask.any(axis=0)
+    return np.array([stats.mode([lab for lab,b in zip(pred_array[:,j],mask[:,j]) if b]).mode[0] if bx else -1 for bx,j in zip(x,range(pred_array.shape[1]))])
+
